
/* 
 * RECView.m -- Implementation file for the RECView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, or that it even has a use.
 *
 */

#import "rec.h"
#import "recdtbl.h"
#import "RECView.h"
#import <NXCType.h>
#import <math.h>
#import <stdlib.h>
#import <string.h>

# define   KK 2
# define CLEN 300	/* buffer length for rec source        */
# define PLEN 900	/* buffer length for rec object        */
# define HMAX  25	/* maximum depth pushdown stack        */
# define PMAX  12	/* maximum polygon accepted by program */
# define  NMU   5	/* number of items in first column     */
# define   NY   8	/* number of demonstration programs    */
# define  NIP  12	/* number of initial polygons          */
# define  NSK   4	/* number of prototype skeletons       */
# define PI 3.14159

//int  rec(char *z);
void refen(int n);

char   cstr[CLEN];	/* console rec program     */
Inst   rprg[PLEN];	/* rec program array       */
int    phil;		/* length of cstr	   */
int    plev;		/* pushdown depth	   */
char   idef[1][30] = { " (()) " };  //(z(!100!0r:;)1r2r(0r:;);) a" };
char   recrule[NY][CLEN] = {
	" ([pentagonal] X5 P CN CN CN CN C p;)",
	" ([bipentag] X5 P CN CN CN Cn Cn Cn Cn C p;)",
	" (TC NC NC NC nC nC nC nC;)"
	};
char   skeletons[NSK][CLEN] = {
	" ([show polygon] O C;)",
	" ([sector] O (! n ! C N:;);)",
	" ([binary] O (! n-1 ! C N:;)(! n-1 ! C n :;);)",
	" ([second] O (! n ! C N:;);)"
	};

int    isErrm=FALSE;
int    drawBack=YES;
int    exSource=YES;
int    options[4]={FALSE,FALSE,FALSE,FALSE};
int    pgon=3, edge=0;
char   errm[50];
double scale=25.0;

/*----------- state color -----------------*/

char dfclr[KK][12] = {"1.0,0.0,0.0","0.0,1.0,0.0"};
struct inpoly {int order; double points[8][3]; char *title;};
struct inpoly inpollies[NIP] = {
		3,{	{ 0.0,  0.0, 0.25},
			{ 5.0,  0.0, 0.25},
			{ 0.0,  8.6, 0.25}},
			"3 - 30-60-90 triangle",
		3,{	{ 0.0,  0.0, 0.25},
			{10.0,  0.0, 0.25},
			{ 0.0, 10.0, 0.25}},
			"3 - 45-45-90 triangle",
		3,{	{ 0.0,  0.0, 0.25},
			{ 5.0,  0.0, 0.25},
			{ 0.0,  8.6, 0.25}},
			"3 - skew triangle",
		4,{	{ 0.0,  0.0, 0.25},
			{10.0,  0.0, 0.25},
			{15.0,  8.7, 0.25},
			{ 5.0,  8.7, 0.25}},
			"4 - rhombus",
		4,{	{ 0.0,  0.0, 0.25},
			{30.0,  0.0, 0.25},
			{20.0, 10.0, 0.25},
			{10.0, 10.0, 0.25}},
			"4 - 45 degree trapezoid",
		4,{	{ 0.0,  0.0, 0.25},
			{20.0,  0.0, 0.25},
			{15.0,  8.6, 0.25},
			{ 5.0,  8.6, 0.25}},
			"4 - 60 degree trapezoid",
		5,{	{ 0.0,  0.0, 0.25},
			{10.0,  0.0, 0.25},
			{10.0,  5.0, 0.25},
			{ 5.0, 10.0, 0.25},
			{ 0.0, 10.0, 0.25}},
			"5 - snub square",
		5,{	{ 0.0,  0.0, 0.5},
			{10.0,  0.0, 0.5},
			{10.0, 10.0, 0.5},
			{ 5.0, 15.0, 0.5},
			{ 0.0, 10.0, 0.5}},
			"5 - 45 gable house",
		5,{	{ 0.0,  0.0, 0.5},
			{10.0,  0.0, 0.5},
			{10.0, 10.0, 0.5},
			{ 5.0, 18.6, 0.5},
			{ 0.0, 10.0, 0.5}},
			"5 - 60 gable house",
		6,{	{ 0.0,  0.0, 0.5},
			{30.0,  0.0, 0.5},
			{30.0, 10.0, 0.5},
			{20.0, 20.0, 0.5},
			{10.0, 20.0, 0.5},
			{ 0.0, 10.0, 0.5}},
			"6 - barn",
		};
			
NXColor  colst[KK];

NXDefaultsVector defaults = { {"well0", "1.0, 0.0, 0.0"},
			      {"well1", "0.0, 1.0, 0.0"},
			      {NULL, NULL} };

char colorwells[KK][6]={"well0","well1"}; 

double high, wide;
double eks, wye, angle, alpha;
double exoff=0.0, wyoff=0.0;
extern double r_dblpar;
double polystack[HMAX][PMAX][3];
double pointbase[PMAX][3],edgebase[PMAX][3];
double pointtraj[PMAX][3],edgetraj[PMAX][3];
double pointsnow[PMAX][3],edgesnow[PMAX][3];
double pointinit[3][3]={{0.0,0.0,1.0}, {0.0,10.0,1.0}, {10.0,0.0,1.0}};
double halftr[3][3]={	{ 0.0,  0.0, 0.25},
			{ 5.0,  0.0, 0.25},
			{ 0.0,  8.6, 0.25}};
double isosrt[3][3]={	{ 0.0,  0.0, 0.25},
			{10.0,  0.0, 0.25},
			{ 0.0, 10.0, 0.25}};
double rhombos[4][3]={	{ 0.0,  0.0, 0.25},
			{10.0,  0.0, 0.25},
			{12.0, 08.0, 0.25},
			{ 5.0,  7.0, 0.25}};
double rhombus[4][3]={	{ 0.0,  0.0, 0.25},
			{10.0,  0.0, 0.25},
			{15.0,  8.7, 0.25},
			{ 5.0,  8.7, 0.25}};
double snubsq[5][3]={	{ 0.0,  0.0, 0.25},
			{10.0,  0.0, 0.25},
			{10.0,  5.0, 0.25},
			{ 5.0, 10.0, 0.25},
			{ 0.0, 10.0, 0.25}};

@implementation RECView

/* ---------------  A P P   I N I T I A L I Z A T I O N   ----------------- */

- appWillInit:sender {printf("appWillInit = REC/F\n"); return self;}

- appDidInit:sender {
 char c[3], fecha[30], tiempo[30], *df;
 int i; 
 double red,green,blue; 
 struct tm  *timeptr;
 time_t  secsnow;
 printf("appDidInit = REC/F for Flexagons\n");

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(fecha,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: fecha];

   sprintf(tiempo,"%02d:%02d:%02d\n",
	timeptr->tm_hour, 
	timeptr->tm_min,
        timeptr->tm_sec);
//   [titime setStringValue: tiempo];

   NXRegisterDefaults("REC/FDefaults",defaults);
   NXUpdateDefaults();
    for(i=0; i <KK; i++) {
     df=NXGetDefaultValue("REC/FDefaults",colorwells[i]);
     if (df==NULL) {
	NXWriteDefault("REC/FDefaults",colorwells[i],dfclr[i]);
	df=&dfclr[i];
	}
     sscanf(df,"%6lf,%6lf,%6lf",&red,&green,&blue); /* don't use %F */
     colst[i]=NXConvertRGBToColor(red,green,blue);
     printf("%s: %s\n",colorwells[i],df);
     }

  [[NXColorPanel sharedInstance:YES] setContinuous: YES];
  [self updateColors];

///* initialize r_vst with select standard procedures */
//   rec_c(idef[0],xprg[0],XLEN,dtbl);
//   r_vst['a']=xprg[0];

  [browser getTitleFromPreviousColumn: YES];
  [browser setDoubleAction: @selector(browserDoubleClick:)];
  [browser setPath: "/demonstrations"];

  [self loadExternal: self];

  return self;
}

- appWillTerminate:sender {printf("goodbye\n"); return self;}

/* ----------------------------- G E N E R A L ----------------------------- */

- saveColors:sender {char c[32]; int i;
    colst[0]=[inCol0 color];
    colst[1]=[inCol1 color];
  for (i=0; i<KK; i++) {
        sprintf(c,"%6.3f, %6.3f, %6.3f",
		NXRedComponent(colst[i]),
		NXGreenComponent(colst[i]),
		NXBlueComponent(colst[i]));
	NXWriteDefault("REC/FDefaults",colorwells[i],c);
	};
return self;
}

- revertColors:sender {char *df; int i; double red, green, blue; 
  for (i=0; i<KK; i++) {
    df=NXGetDefaultValue("REC/FDefaults",colorwells[i]);
       if (df==NULL) {printf("null df = %s\n",df); return self;}
       sscanf(df,"%6lf,%6lf,%6lf",&red,&green,&blue); 
    colst[i]=NXConvertRGBToColor(red,green,blue);
    printf("%s: %s\n",colorwells[i],df);
    }
  [self updateColors];
return self;
}

- updateColors {
printf("updateColors\n");
	[inCol0 setColor: colst[0]];
	[inCol1 setColor: colst[1]];
return self;
}

//- windowDidBecomeKey:sender {
//	[menuCopy setTarget: self];
//	[menuCopy setAction: @selector(copyWells:)];
//	[menuCopy setEnabled: YES];
//return self;
//}

//- windowDidResignKey:sender {[menuCopy setEnabled: NO]; return self;}

- copyWells:sender {
NXRect zbounds;
id     pBoard;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theWellBox getBounds: &zbounds];
  [theWellBox writePSCodeInside: &zbounds to: pBoard]; 
return self;
}

/* = = = = = = =   P a r a m e t e r   A d j u s t m e n t   = = = = = = = */

- rexsli:sender {int i;					/* set x-offset */
	exoff=120.0*[sender doubleValue];
 	[bigView display];
return self;
}

- rwysli:sender {					/* set y-offset */
	wyoff=120.0*[sender doubleValue];
 	[bigView display];
return self;
}


/* = = = = = = = = = =   P r o g r a m   E d i t o r   = = = = = = = = = = */

- recMatrix:sender { printf("selected row = %3d\n ",[sender selectedRow]);
  [comment setStringValue: 
	dtbl[ [sender selectedRow] * 32 + [sender selectedCol] ].r_cmnt];
return self;
}

- ascii:sender {
  dtbl[ [sender selectedRow] * 32 + [sender selectedCol] ].xfun();
return self;
}

/* = = = = = = = = =   P r o g r a m   E x e c u t o r   = = = = = = = = = */

- setOption:sender {int i;
  i=[sender selectedCol];
  options[i]=[[sender selectedCell] state];
return self;
}

- run:sender {[lilView display]; return self;}

- recompile:sender {
  [execWin getSubstring: cstr start: 0 length: [execWin textLength]];
  [window setDocEdited:YES];
return self;
}

- browserDoubleClick:sender {int i, j, k; 
   if (strcmp([browser titleOfColumn: [browser selectedColumn]],
	"initial polygons") == 0) {
	k=[[sender matrixInColumn: [sender selectedColumn]] selectedRow];
	pgon=inpollies[k].order;
	edge = 0;
	for (i=0; i<pgon; i++) for (j=0; j<3; j++) 
		pointbase[i][j] = inpollies[k].points[i][j];
   	[execLine setStringValue: "(CP;)"];
  	strcpy(cstr,"(CP;)");}
   else if (strcmp([browser titleOfColumn: [browser selectedColumn]],
	"demonstrations") == 0) {
   	[value setStringValue: " "];
   	[execLine setStringValue: [sender stringValue]];
  	strcpy(cstr,[sender stringValue]);
	}
   else if (strcmp([browser titleOfColumn: [browser selectedColumn]],
	"skeletons") == 0) {
   	[value setStringValue: " "];
   	[execLine setStringValue: [sender stringValue]];
	}
   [lilView display];
return self;
  }

- gho:sender {
   strcpy(cstr,[sender stringValue]);
printf("went and ghoed: %s\n",[sender stringValue]);
printf("went and ghoed: %s\n",cstr);
   [lilView display];
   return self;
  }

/* = = = = = = = = = = = =   B i g   W i n d o w   = = = = = = = = = = = = */

- openBigWin:sender {
  exoff = 0.0; wyoff = 0.0;
  [bigWindow makeKeyAndOrderFront: sender];
return self; 
}

- smartPrintBigWin:sender {return self;}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/*             definition of REC operators and predicates                  */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

int rec(z) char *z; {if (rec_c(z,rprg,PLEN,dtbl)==1) rec_x(rprg);}

int rprze() {return(options[0]==TRUE);}
int rpron() {return(options[1]==TRUE);}
int rprtw() {return(options[2]==TRUE);}
int rprth() {return(options[3]==TRUE);}

void ropla() {alpha=(r_dblpar*PI)/180.0;}		/* set alpha     */

void roplc() {						/* draw circle   */
int c;
				PSstroke();
	c = *r_pc++;
	switch (c) {
	  case 'r': PSsetrgbcolor(1.0,0.0,0.0); break;
	  case 'g': PSsetrgbcolor(0.0,1.0,0.0); break;
	  case 'b': PSsetrgbcolor(0.0,0.0,1.0); break;
	  case 'k': PSsetgray(NX_BLACK); break;
	  case 'c': PSsetcmykcolor(1.0,0.0,0.0,0.0); break;
	  case 'm': PSsetcmykcolor(0.0,1.0,0.0,0.0); break;
	  case 'y': PSsetcmykcolor(0.0,0.0,1.0,0.0); break;
default:  PSsetgray(NX_BLACK); break;}
				PSmoveto(eks+5.0,wye);
				PSarc(eks, wye, 5.0, 0.0, 360.0);
				PSmoveto(eks,wye);
				PSstroke();
				PSsetgray(NX_BLACK);
}

void roplf() {}						/* shorten step    */

void roplh() {scale*=0.5;}				/* halve step      */

void roplj() {}						/* jog angle       */

void ropll() {}						/* turn leftward   */

void roplm() {	}					/* take step       */

void ropln() {edge--; refen((edge+pgon) % pgon);}  /* reflect in previous edge */

void roplo() {int i;				      /* offset the polygon */
    for (i=0; i<pgon; i++)  
	pointbase[i][0]=pointbase[i][0]/pointbase[i][2]+20.0;
	pointbase[i][1]=pointbase[i][1]/pointbase[i][2];
	pointbase[i][2]=1.0;
}

void roplp() {						/* pop fpolygon */
	if (plev<0) return;
	roplo();
	plev--;
}

void roplr() {}						/* turn rightward */

void ropls() {scale*=r_dblpar;}				/* change scale   */



void ropua() {angle=(r_dblpar*PI)/180.0;}		/* set angle   */

void ropuc() {int i;					/* draw polygon */
    PSsetgray(NX_BLACK);
    PSmoveto(pointbase[pgon-1][0]/pointbase[pgon-1][2]+exoff,
    	pointbase[pgon-1][1]/pointbase[pgon-1][2]+wyoff);
    for (i=0; i<pgon; i++) {
    	PSlineto(pointbase[i][0]/pointbase[i][2]+exoff,
            pointbase[i][1]/pointbase[i][2]+wyoff);}
}

void ropuf() {}						/* lengthen step     */

void ropuh() {scale*=2.0;}				/* double step       */

void ropuk() {int i, j;					/* turn 60 deg right */
    pgon = 3;
    edge = 0;
    for (i=0; i<pgon; i++) for (j=0; j<3; j++) pointbase[i][j] = isosrt[i][j];   
}				

void ropul() {int i, j;					/* turn 60 deg right */
    pgon = 3;
    edge = 0;
    for (i=0; i<pgon; i++) for (j=0; j<3; j++) pointbase[i][j] = halftr[i][j];   
}				

void ropum() {	}					/* take step         */

void ropun() {edge++; refen(edge % pgon);}	     /* reflect in next edge */

void ropuo() {int i, j;				        /* read top  polygon */
    for (i=0; i<pgon; i++) for (j=0; j<3; j++)   
	pointbase[i][j]=polystack[plev][i][j];
}

void ropup() {int i, j;					     /* push polygon */
	plev++;
	if (plev>=HMAX) return;
    for (i=0; i<pgon; i++) for (j=0; j<3; j++)   
	polystack[plev][i][j]=pointbase[i][j];
}

int  rpruq() {return(scale*high<5.0);}		/* test for minimum scale */

void refen(nn) int nn; {			     /* reflect in edge n */
    int i, m, n;
    double aa, bb, cc, x, y, ex1, ex2, wy1, wy2, rr, m11, m12, m21, m22;
	n = nn % pgon;
	m = (n + 1) % pgon;
	ex1 = pointbase[n][0]/pointbase[n][2];
	ex2 = pointbase[m][0]/pointbase[m][2];
	wy1 = pointbase[n][1]/pointbase[n][2];
	wy2 = pointbase[m][1]/pointbase[m][2];
	aa = wy1 - wy2; 
	bb = ex2 - ex1;
	cc = wy2 * ex1 - wy1 * ex2;
	rr = aa * aa + bb * bb;
	m11 =  bb * bb - aa * aa;
	m12 = -2.0 * aa * bb;
	m21 =  m12;
	m22 = -m11;
	for (i=0; i<pgon; i++) {
	    x = (m11 * pointbase[i][0] + m12 * pointbase[i][1])
		/pointbase[i][2];
	    y = (m21 * pointbase[i][0] + m22 * pointbase[i][1])
		/pointbase[i][2];
	    pointbase[i][0] = (x - 2.0 * aa * cc)/rr;
	    pointbase[i][1] = (y - 2.0 * bb * cc)/rr;
	    pointbase[i][2] = 1.0;
	    }
//for (i=0; i<pgon; i++) printf("%6.3f, %6.3f, %6.3f, \n",
//	pointbase[i][0],pointbase[i][1],pointbase[i][2]);
//	printf("\n");
}	

void ropur() {int n;		      		        /* reflect in edge n */
	n = *r_pc++ - '0';
	if (n < 0 || n > pgon-1) return; else {edge=n; refen(n);};
}

void ropus() {scale=r_dblpar;}					/* set scale */

void roput() {int i, j;						/* rhombus */
    pgon = 4;
    edge = 0;
    for (i=0; i<pgon; i++) for (j=0; j<3; j++) pointbase[i][j]=rhombus[i][j];
}				

void ropuu() {int i, j;					      /* snubsquare */
    pgon = 5;
    edge = 0;
    for (i=0; i<pgon; i++) for (j=0; j<3; j++) pointbase[i][j]=snubsq[i][j];
}				

void ropux() {int c, i; double th, dth;			  /* regular polygon */
	c = *r_pc++ - '0';
	if (c < 1 || c > 12) return;
	pgon = c;
	edge = pgon-1;
	dth = (2 * PI)/((double)(pgon));
	for (i=0, th=-0.5*dth; i<pgon; i++, th+=dth) {
		pointbase[i][0] = cos(th);
		pointbase[i][1] = sin(th);
		pointbase[i][2] = 1.0/scale;
		}
}

void ropuz() {PSerasepage();}

/* ------------ */

void ropqm() {						/* print reg values */
  printf("scale %6.3f,plev %3d\n",scale,plev);
}

int rprns() {return (int)(random()&01);}		/* random 1/2 pred */

int rprps() {return random()%10==1;}			/* random 1/10 pred */

/* - - - - - - - -   F I L E   A N D   D I R E C T O R Y   - - - - - - - - */

- setFilename:(const char *)aFilename {
if (filename) free(filename);
filename = malloc(strlen(aFilename)+1);
strcpy(filename, aFilename);
[window setTitleAsFilename:aFilename];
return self;
}

- open:sender {
  char *types[2] = {"rec",0};
  int  i, fd;
  NXStream *theStream;

  [[OpenPanel new] allowMultipleFiles: NO];
  [[OpenPanel new] chooseDirectories: NO];

  if ([ [OpenPanel new] runModalForTypes:types]) {
    [self setFilename: [[OpenPanel new] filename]];
    fd = open(filename, O_RDONLY, 06666);
    theStream = NXOpenFile(fd, NX_READONLY);
//      NXScanf(theStream,"%s",cstr);
      i=0;
      while (NXScanf(theStream,"%c",&cstr[i])!=EOF) i++;
      phil=strlen(cstr);
    NXClose(theStream);
    close(fd);
    [window setTitleAsFilename: filename];
    [window setDocEdited:NO];
  [execWin setSel: 0: [execWin textLength]];
  [execWin replaceSel: cstr];
  [execWin scrollSelToVisible];
  [execWin display];
    }
return self;
}

- saveAs:sender {
  id panel;
  const char *dir;
  char *file;

/* prompt user for the file name and save to that file */

  if (filename==0) {
    /* no filename; set up defaults */ 
    dir  = NXHomeDirectory();
    file = (char *)[window title];
    } else {
    file=rindex(filename,'/');
    if (file) {
      dir   = filename;
      *file = 0;
      file++;
      } else {
      dir  = filename;
      file = (char *)[window title];
      }
    }
  panel = [SavePanel new]; 
  [panel setRequiredFileType: ""];
  if ([panel runModalForDirectory: dir file: file]) {
    [self setFilename: [panel filename]];
    return [self save: sender];
    }
    return nil;  /* didn't save */
  }
    
- save:sender {
  int      fd;
  NXStream *theStream;

  if (filename==0) return [self saveAs: sender];
  [window setTitle: "Saving ..."];

  fd = open(filename, O_WRONLY | O_CREAT | O_TRUNC, 06666);
  if (fd < 0) {
    NXRunAlertPanel(0, "Cannot save file: %s",0,0,0,strerror(errno));
    return self;
    }
  theStream = NXOpenFile(fd, NX_WRITEONLY);
  [execWin writeText: theStream];
  NXClose(theStream);
  close(fd);
  [window setTitleAsFilename: filename];
  [window setDocEdited:NO];
  return self;
}

- close:sender {const char *fname;
int         q;
  if ([window isDocEdited])
//    fname=filename ? filename : [sender title];
//    if (rindex(fname,'/')) fname = rindex(fname,'/') + 1;
    q = NXRunAlertPanel(
      "Save",
      "Save changes to %s?",
      "Save",
      "Don't Save",
      "Cancel",
      filename);
    if (q==1) {if (![self save:nil]) return nil;}
    if(q==-1) return nil;
//  [sender setDelegate: nil];
//  [proc free];
//  [self free];
  return self;
}

- windowWillClose:sender {
[self close: self];
printf("windowWillClose\n");
return self;
}

/* - - - - - - -   c o p y   a n d   p r i n t   v i e w s   - - - - - - */

- printREC:sender {[keyWindow smartPrintPSCode: sender]; return self;}

- windowDidBecomeKey:sender {
	keyWindow=sender;
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copyREC:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printREC:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copyREC:sender {
    drawBack=NO;

    if (keyWindow==bigWindow)
	[self copySomeView: bigView];
    if (keyWindow==infoPanel)
	[self copySomeView: [infoPanel contentView]];
    if (keyWindow==execWin)
	[self copySomeView: [execWin contentView]];
    if (keyWindow==parPanel)
	[self copySomeView: [parPanel contentView]];

    drawBack=YES;
return self;
}

- copySomeView:theView {
NXRect zbounds;
id     pBoard;
//  drawView=viewnum;
//  *draw=YES;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [theView getBounds: &zbounds];
  [theView writePSCodeInside: &zbounds to: pBoard]; 
//  *draw=NO;
return self;
}

/*---------------- B R O W S E R  D E L E G A T E -------------------*/

- (int)browser:sender fillMatrix: matrix inColumn: (int)col { int  i, n;
    n=0;
    printf("browser\n");
    switch(col) {
      case 0: 
        for(i=0; i<NMU; i++) {
          [matrix addRow];
          [[matrix cellAt: i: 0]  setLeaf : NO ];     
          [[matrix cellAt: i: 0]  setLoaded : YES ];   
          [[matrix cellAt: i: 0]  setEnabled : YES ];
          [[matrix cellAt: i: 0]  setEnabled : YES ];
          }
        [[matrix cellAt: 0: 0]  setStringValue: "operators"];     
        [[matrix cellAt: 1: 0]  setStringValue: "predefined"];     
        [[matrix cellAt: 2: 0]  setStringValue: "skeletons"];
        [[matrix cellAt: 3: 0]  setStringValue: "initial polygons"];
        [[matrix cellAt: 4: 0]  setStringValue: "demonstrations"];
        n=NMU;
        break;
      case 1:
        if(strcmp([[sender selectedCell] stringValue], "demonstrations") == 0){
          for(i=0; i<NY; i++) {
            [matrix addRow];
            [[matrix cellAt: i: 0]  setStringValue: recrule[i]];
            [[matrix cellAt: i: 0]  setLeaf : YES ];     
            [[matrix cellAt: i: 0]  setLoaded : YES ];   
            [[matrix cellAt: i: 0]  setEnabled : YES ];
            }
          n=NY;}

        if(strcmp([[sender selectedCell] stringValue], "predefined") == 0){
          for(i=0; i<1; i++) {
            [matrix addRow];
            [[matrix cellAt: i: 0]  setStringValue: idef[i]];
            [[matrix cellAt: i: 0]  setLeaf : YES ];     
            [[matrix cellAt: i: 0]  setLoaded : YES ];   
            [[matrix cellAt: i: 0]  setEnabled : YES ];
            }
          n=1;}

        if(strcmp([[sender selectedCell] stringValue], "operators") == 0){
          for(i=0; i<95; i++) {
            [matrix addRow];
            [[matrix cellAt: i: 0]  setStringValue: dtbl[i].r_cmnt];
            [[matrix cellAt: i: 0]  setLeaf : YES ];     
            [[matrix cellAt: i: 0]  setLoaded : YES ];   
            [[matrix cellAt: i: 0]  setEnabled : YES ];
            }
          n=95;}
        if(strcmp([[sender selectedCell] stringValue], "initial polygons") == 0){
           for(i=0; i<NIP; i++) {
            [matrix addRow];
            [[matrix cellAt: i: 0]  setStringValue: inpollies[i].title];
            [[matrix cellAt: i: 0]  setLeaf : YES ];     
            [[matrix cellAt: i: 0]  setLoaded : YES ];   
            [[matrix cellAt: i: 0]  setEnabled : YES ];
           }
          n=NIP;}
        if(strcmp([[sender selectedCell] stringValue], "skeletons") == 0){
           for(i=0; i<NIP; i++) {
            [matrix addRow];
            [[matrix cellAt: i: 0]  setStringValue: skeletons[i]];
            [[matrix cellAt: i: 0]  setLeaf : YES ];     
            [[matrix cellAt: i: 0]  setLoaded : YES ];   
            [[matrix cellAt: i: 0]  setEnabled : YES ];
           }
          n=NSK;}

            break;
        default: n=0; break; }
      return n;
    }

/* to distract the single click in a browser double click */
- nhul: sender {return self;}

/*******   REC program management ********/

- loadRECPanel:sender {
	if ([sender state]) [self loadInternal: sender]; 
	else                [self loadExternal: sender];}

- loadInternal:sender {
  printf("Internal\n ");
  [self loadLowerPaneltwo: internalRECBox ];
  return self;
  }

- loadExternal:sender { 
  printf("External\n ");
  [self loadLowerPaneltwo: externalRECBox ];
  return self;
  }

- loadLowerPaneltwo:thisRuleBox {NXRect bxRect, loRect;
  [theBigBox getFrame: &bxRect];
  [thisRuleBox getFrame: &loRect];
  [theLilBox setContentView: [thisRuleBox contentView]];
  loRect.origin.x=(bxRect.size.width-loRect.size.width)/2.0;
  loRect.origin.y=(bxRect.size.height-loRect.size.height)/2.0;
  [theLilBox setFrame: &loRect];
  [theBigBox display];
  [theLilBox display];
return self;
}

- textDidChange:textObject {
  return  [windows  setDocEdited: YES];
} 

/* = = = = = = = = = = = = = =   R E C V I E W   = = = = = = = = = = = = = */

- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new RECView object.  First, an initFrame: message is sent 
 * to super to initialize RECView as a View.  Next, the RECView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system 
 * lies in the center of its area.  
 */
{	
	[super initFrame:frameRect];
	[self setOpaque:YES];
	[self translate:floor(frame.size.width/2) :floor(frame.size.height/2)];
	return self;
}

- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the RECView's background and axes.  If there are any points,
 * these are drawn too.
 */
{	
	if (rects == NULL) return self;

//	PSsetgray(NX_WHITE);
	PSsetgray(NX_LTGRAY);
	if (drawBack) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);
//	PSsetgray(NX_BLACK);

		    isErrm=FALSE;
		    [value setStringValue: " "];
		    if (rec(cstr)) [value setStringValue: "T"];
		        else [value setStringValue: "F"];
		    if (isErrm) {[comment setStringValue: errm]; isErrm=FALSE;}
		    PSstroke();

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the RECView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
	[super sizeTo:width :height];
	[self setDrawOrigin:-floor(width/2) : -floor(height/2)]; 

	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the 
 * mouse button when the cursor is over the RECView.  The RECView changes 
 * the cursor to a cross-hairs image and then starts asking for mouse-dragged 
 * or mouse-up events.  As it receives mouse-dragged events, the RECView 
 * updates the readOut text Cell with the cursor's coordinates.  If the user 
 * releases the mouse button while the cursor is over the RECView, the RECView 
 * registers the point and then sends a message to its delegate notifying it 
 * of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
	NXRect	plotRect;

	[crossCursor set];
   	[self getBounds:&plotRect];

	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
	[self lockFocus];
	do {
		aPoint = theEvent->location;
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {
	/* on mouse-up, register point, inform delegate, and clean up state */
			[window flushWindow];
			looping = NO;
		}	
   	} while (looping && 
    (theEvent=[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];
	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

@end 

/* end RECView.m */